
#' algdoor

#' algdoor
#' @param service service
#' @keywords algoli
#' @export
#' @examples
#' \dontrun{
#' algdoor()
#' }
algdoor <- function(service){
  algdoor_object(service=service)
}

algdoor_object <- function(service){

  service_ <- service

  ########################################
  # function related to remote libraries #
  ########################################
  core_algdoor <- function(){
    fun <- try(utils::getFromNamespace(paste0(service_,"_core"),service_),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor core function not found")
    } else {
      res <- do.call("fun",list())
      return(res)
    }
  }

  connect_algdoor <- function(){
    fun <- try(utils::getFromNamespace("connect",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor model function not found")
    } else {
      if (is.null(sc_)){
         sc_ <<- do.call("fun",list())
      } else stop("connection done already")
    }
    invisible()
  }

  disconnect_algdoor <- function(){
    fun <- try(utils::getFromNamespace("disconnect",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor model function not found")
    } else {
      if (!is.null(sc_)){
         do.call("fun",list(sc_))
      } else stop("no existing connection")
    }
    invisible()
  }

  df_algdoor <- function(x,name,as_factor){
    fun <- try(utils::getFromNamespace("df",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor model function not found")
    } else {
      res <- do.call("fun",list(sc=sc_,x=x,name=name,as_factor=as_factor))
      return(res)
    }
  }

  copy_algdoor <- function(dataset){
    fun <- try(utils::getFromNamespace("copy",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor model function not found")
    } else {
      res <- do.call("fun",list(sc=sc_,dataset=dataset))
      return(res)
    }
  }

  split_algdoor <- function(dataset,train,test,seed){
    fun <- try(utils::getFromNamespace("split_d",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor model function not found")
    } else {
      res <- do.call("fun",list(dataset,train=train,test=test,seed=seed))
      return(res)
    }
  }

  export_model_algdoor <- function(model,path){
    fun <- try(utils::getFromNamespace("export_model",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor export_model function not found")
    } else {
      res <- do.call("fun",list(model=model,path=path))
      return(res)
    }
  }

  load_model_algdoor <- function(path){
    fun <- try(utils::getFromNamespace("load_model",core_algdoor()),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor load_model function not found")
    } else {
      res <- do.call("fun",list(path=path))
      return(res)
    }
  }

  fit_algdoor <- function(algorithm,target,predictors,...){
    if (!is.null(train_)) {
      fun <- try(utils::getFromNamespace(paste0(service_,"_fit_",algorithm),service_),silent=TRUE)
      if (inherits(fun,"try-error")==TRUE) {
        stop("algdoor model function not found")
      } else {
        res <- do.call("fun",list(target=target,predictors=predictors,train=train_,...))
        return(res)
      }
    } else stop ("Train dataset is missing")

  }

  predict_algdoor <- function(model,newdata=NULL){

    fun <- try(utils::getFromNamespace(paste0(service_,"_predict_",model$algorithm),service_),silent=TRUE)
    if (inherits(fun,"try-error")==TRUE) {
      stop("algdoor predict function not found")
    } else {
      if (!is.null(newdata)){
        if (!is.null(col_bind_)){
          res <- cbind(subset(newdata,select=c(col_bind_)),do.call("fun",list(sc=sc_,model=model,newdata=newdata)) )
        } else {
           res <- do.call("fun",list(sc=sc_,model=model,newdata=newdata))
        }
      } else {
        if (!is.null(col_bind_)){
          res <- cbind(subset(test_,select=c(col_bind_)),do.call("fun",list(sc=sc_,model=model,newdata=test_)) )
        } else {
          res <- do.call("fun",list(sc=sc_,model=model,newdata=test_))
        }
      }
      return(res)
    }
  }

  ###############################
  # pointers to variables in sc #
  ###############################
  sc_ <- NULL
  train_ <- NULL
  test_ <- NULL
  col_bind_ <- NULL


  ######################
  # internal functions #
  ######################
  get_col_bind <- function(col_bind){
    col_bind_ <<- col_bind
    invisible()
  }

  data_train <- function(x,as_factor){
    train_<<- df_algdoor(x=x,name="train_",as_factor=as_factor)
    invisible()
  }

  data_test <- function(x,as_factor){
    test_  <<- df_algdoor(x=x,name="test_",as_factor=as_factor)
    invisible()
  }


  # public
  object <- local({

    connect <- function(){
      connect_algdoor()
      invisible()
    }

    disconnect <- function(){
      disconnect_algdoor()
      invisible()
    }

    dataset <- function(dataset,col_bind=NULL,train=80,test=20,seed=NULL,as_factor=NULL){
      get_col_bind(col_bind)
      tmp <- split_algdoor(copy_algdoor(dataset) ,train=train,test=test,seed=seed)
      if (train!=0L) data_train(x=tmp$train,as_factor=as_factor)
      data_test(x=tmp$test,as_factor=as_factor)
      invisible()
    }

    fit <- function(algorithm,target,predictors,...){
      res <- fit_algdoor(algorithm=algorithm,target=target,predictors=predictors,...)
      return(res)
    }

    predict <- function(model,newdata=NULL){
      res <- predict_algdoor(model=model, newdata=newdata)
      return(res)
    }

    export_model <- function(model,path){
      res <- export_model_algdoor(model=model,path=path)
      return(res)
    }

    load_model <- function(path){
      res <- load_model_algdoor(path=path)
      return(res)
    }

    summary <- function(){
      print(paste0("Algorithms from ",service_))
    }

    environment()
    })
    lockEnvironment(object, TRUE)
    structure(object, class=c("algdoor", class(object)))
}
